# -*- coding: utf-8 -*-
"""
Created on Tue Jan 29 15:31:20 2019

@author: Luciano A. Masullo (Laboratorio 2, 1er Cuatrimestre 2019, DF-FCEyN-UBA)

En este script se muestra como cargar datos de una medición con sus errores,
graficarlos y realizar un ajuste lineal con una función definida por el
usuario.

"""

# en cualquier código primero van los imports de las librerías que se van a usar

import numpy as np   # numpy es la librería para cálculo numérico
import matplotlib.pyplot as plt  # matplotlib es una librería para graficar
from scipy.optimize import curve_fit

π = np.pi  # defino la constante π

# por convención al principio del código se definen las funciones que se van a usar
# también se podría escribir otro archivo con las funciones e importarlas
# TAREA: probar escribir un archivo funciones.py con la funcion_lineal e importarla

def funcion_lineal(t, m, q):
    
    y = m*t + q
    
    return y

# se cargan los datos

data = np.loadtxt('datos_lineal.txt', skiprows=2)

# se muestra por pantalla la dimensión de la matriz, ¿Tiene sentido? ¿Son mis datos?
# se puede también mostrar por pantalla los datos o sus primeros elementos, etc
# TAREA: probar mostrar (imprimir) por pantalla los primeros 10 pares de datos

print('La variable "data" es una matriz de dimensiones', np.shape(data))

y = data[:, 0]
σ_y = data[:, 1]
x = data[:, 2]

# se grafican los datos, hay definidos sólo algunos parámetros del gráfico
# TAREA: cambiar el color y el tamaño de los puntos

plt.figure('Mediciones de ejemplo')
plt.xlabel('Tiempo (ms)')
plt.ylabel('Amplitud (V)')
plt.errorbar(x, y, yerr=σ_y, fmt='ko', capsize=5)
plt.grid(True)

# cruve_fit es la función que hace el ajuste de la función modelo a los datos  (x, y)

popt, pcov = curve_fit(funcion_lineal, x, y)
perr = np.sqrt(np.diag(pcov))  # esta línea calcula los errores de los parámetros "perr"
                               # del ajuste a partir de la matriz de covarianza "pcov"

# guardamos los resultados del ajuste

m_fit = popt[0]
q_fit = popt[1]

σ_m = perr[0]
σ_q = perr[1]

# mostramos por pantalla los valores del ajuste

print('m =', m_fit, '+/-', σ_m)
print('q =', q_fit, '+/-', σ_q)

# TAREA: ¿Cuántas cifras tienen los parámetros y los errores? Busque cómo hacer para
# redondear valores usando una función de NumPy y redondee los valores a 2 decimales 

# generamos la curva que vamos a graficar (¡Ojo, el ajuste devolvió dos parámetros no una curva!)

ajuste = funcion_lineal(x, m_fit, q_fit)

# graficamos la curva sobre los datos
# TAREA: cambiar el color del gráfico y graficar con una línea punteada

plt.plot(x, ajuste, 'r-', linewidth=2.0)
