# -*- coding: utf-8 -*-
"""
Created on Tue Jan 29 15:31:20 2019

@author: Luciano A. Masullo (Laboratorio 2, 1er Cuatrimestre 2019, DF-FCEyN-UBA)

En este script se muestra como cargar datos de una medición con sus errores,
graficarlos y realizar un ajuste no-lineal con una función definida por el
usuario.

"""

import numpy as np   # numpy es la librería para cálculo numérico
import matplotlib.pyplot as plt  # matplotlib es una librería para graficar
from scipy.optimize import curve_fit

π = np.pi  # defino la constante π

def funcion_seno(t, A, ω, Φ, y0):
    
    y = A * np.sin(ω*t + Φ) + y0
    
    return y

# se cargan los datos

data = np.loadtxt('datos_nolineal.txt', skiprows=2)

print('La variable "data" es una matriz de dimensiones', np.shape(data))

y = data[:, 0]
σ_y = data[:, 1]
x = data[:, 2]

plt.figure('Mediciones de ejemplo')
plt.xlabel('Tiempo (ms)')
plt.ylabel('Amplitud (V)')
plt.errorbar(x, y, yerr=σ_y, fmt='ko', capsize=5)
plt.grid(True)

# Para un ajuste no lineal es escencial hacer una buena elección de parámetros iniciales
# TAREA: probar qué pasa si se borra el campo "p0=init_guess"

A_i = 2.0

T_i = 20.0
f_i = 1/T_i
ω_i = (2*π)* f_i

Φ_i = 0.0

y0_i = 0.5

init_guess = [A_i, ω_i, Φ_i, y0_i]

# cruve_fit es la función que hace el ajuste de la función modelo a los datos  (x, y)

popt, pcov = curve_fit(funcion_seno, x, y, p0=init_guess)
perr = np.sqrt(np.diag(pcov))  # esta línea calcula los errores de los parámetros "perr"
                               # del ajuste a partir de la matriz de covarianza "pcov"

# guardamos los resultados del ajuste

A_fit = popt[0]
ω_fit = popt[1]
Φ_fit = popt[2]
y0_fit = popt[3]

σ_A = perr[0]
σ_ω = perr[1]
σ_Φ = perr[2]
σ_y0 = perr[3]

# mostramos por pantalla los valores del ajuste

print('A =', A_fit, '+/-', σ_A)
print('ω =', ω_fit, '+/-', σ_ω)
print('Φ =', Φ_fit, '+/-', σ_Φ)
print('Φ =', y0_fit, '+/-', σ_y0)

# para graficar el modelo hacemos un array de abscisas (xx) de más puntos que los datos
# ajuste es el array de evaluar la funcion_seno sobre este array xx

xmin = 0
xmax = 60
xx = np.linspace(xmin, xmax, num=1000)
ajuste = funcion_seno(xx, A_fit, ω_fit, Φ_fit, y0_fit)

plt.plot(xx, ajuste, 'r-', linewidth=2.0, zorder=5)
